function [ind,w,xHOC,lb] = HOC(x,A,b,Sx,h)
  % function [ind,w,xHOC,lb] = HOC(x,A,b,Sx,h)
% =========================================================================
%  FILE:           HOC.m            % HOC = Heuristic Optimality Check
%  AUTHOR:         Andreas M. Tillmann, TU Darmstadt, Germany, Germany                  
% =========================================================================
%  DESCRIPTION:    This function (heuristically) evualuates the optimality 
%                  condition of the L1-minimization problem
%                      min ||x||_1  such that  Ax=b,
%                  i.e., that there is a vector w of length m row dim. 
%                  of A) such that (-)A'*w is in the subdifferential of the 
%                  l1-norm at the point x. The optimality criterion can be 
%                  formulated as the problem of finding a vector w such 
%                  that
%                      A(:,Sx)^T * w = -sign(x(Sx)) 
%                                      (on the (approx.) support Sx of x),
%                      -1 <= (A^T * w)_i <= 1    for all i=1,...,n
%                  HOC ignores the second constraint when computing w, 
%                  which is done here by calculating an (approximate) 
%                  solution to the normal equation 
%                      A(:,Sx)*A(:,Sx)^T *w = -A(:,Sx)*sign(x(Sx))
%                  induced by the first (sign-)constraint. Note that in 
%                  particular, HOC does not check overall existence of a w
%                  satisfying the above constraints, but looks for a speci-
%                  fic vector w; see remark below. Also, when w obeys the
%                  above conditions, HOC solves for xHOC on the support Sx,
%                  i.e., tries to establish A*xHOC=A(:,Sx)*xHOC(:,Sx)=b.
%                  The hope here is that the sign patterns of x(:,Sx) and
%                  xHOC(:,Sx) are the same, but this is not verified.
%
%                  Due to its heuristic nature, HOC may possibly lead to a
%                  wrong indication of optimality, which is very effectively
%                  reduced by additionally evaluating the (relative) duality
%                  gap corresponding to the computed dual vector w and the
%                  primal solution supported on Sx. 
%
%                  Remark: Empirically, the HOC was found to work well 
%                  when the solution x is the unique l1-minimizing solution
%                  to Ax=b and there are no off-support entries equal to 
%                  +/-1 in the subgradient A'w computed here. Otherwise, 
%                  HOC may fail to recognize optimality, since in that 
%                  case, the normal equation approach used here will not 
%                  necessarily work (see Fuchs (2004)). We have never ob-
%                  served that a wrong vector was returned as optimal.
%                  Moreover, when the approximate support is a superset of
%                  the true optimal support and induces a full-rank sub-
%                  matrix ASx, HOC also worked well in our experiments.
%
%                  Details on HOC can be found in the paper
%                  "Solving Basis Pursuit: Heuristic Optimality Check
%                   and Solver Comparison", 
%                  by D.A.Lorenz, M.E.Pfetsch and A.M.Tillmann, 2011-13,
%                  Optimization Online eprint-ID 2011-07-3100.
%                  If a reference is needed, please use this article.
%
%                  (This HOC version was taken from l1testpack v.1.2)
%
%
%  INPUT:          x    The vector whose subdifferential is to be evaluated
%                  A    The coefficient matrix of the constraint Ax=b
%                  b    The corresponding right hand side vector
%                  Sx   Approx. support of vector x (index vector)
%                       [ optional; if argument Sx is not passed, then Sx
%                         will be supp(x,1e-9) by default ]
%                  h    A subgradient of ||.||_1 at x
%                       [ optional; default is h = sign(x) ]
%
%  OUTPUT:         ind  Optimality indicator, a value of 1 indicates
%                       that the provided x is an optimal solution of
%                       min ||x||_1 s.t. Ax=b
%                  w    The dual variable for which A'w is hopefully a
%                       subgradient of ||.||_1 at x
%                  xHOC If ind=0, this is just the input x, otherwise it
%                       is the (approx.) optimal vector, having support Sx 
%                  lb   A lower bound on the optimal l1-value, obtained
%                       from w (scaled to be dual-feasible) as -b'*w
% =========================================================================
% $Revision: 260 $
% $Author: andtillm $
% $Date: 2011-07-15 08:30:28 +0200 (Fr, 15. Jul 2011) $
% =========================================================================
if( nargin < 4 )
    Sx = supp(x,1e-9);  % default support approximation: ignore entries 
end                     % with absolute value smaller than 1e-9
if( nargin < 5 )       
    h = sign(x);        % default subgradient: sign-vector of x
end

% calculate approx. solution to A[sx]'w = -sign(x(sx)) :
ASx = A(:,Sx);
w = ASx*CG( ASx', [], -h(Sx), 1e-9, 20, zeros(numel(Sx),1) );

% hoping that, in an (approx.) optimal x, the solution w which satisfies 
% the constraint "A'*w in [-1,1]" (A'*w in subdiff. to l1-norm; ignored above) 
% was actually found ... :
temp = norm(A'*w,'inf');
% unless b=0 in the original problem, x_opt will always have at least one
% nonzero entry, hence any subgradient to an optimal point must have at 
% least one entry equal to +/- 1 !

ind = 0;  % HOC cannot confirm optimality
xHOC = x;
 
if( (temp < 1+1e-6) && (temp > 1-1e-6) ) % allow errors/infeas. up to 1e-6    
    xHOC = zeros(size(x));    
    xHOC(Sx) = ASx\b;                    % solve for xHOC on support Sx
    if( norm(A*xHOC-b,'inf')<1e-6 )      % i.e., xHOC is (approx) feasible
       lb = -(b'*w)/temp;                % corresp. dual objective value
       xnrm = norm(xHOC,1);              % primal objective value
       if( abs( xnrm - lb )/xnrm < 1e-6) % small relative duality gap
          ind = 1;       % optimality condition (approximately) satisfied
       end
    end
else
    if( nargout > 3 )      % compute dual lower bound on primal optimal value
        lb = -(b'*w)/temp; % w/temp is a feasible dual solution, even if it does 
    end                    % not yield a subgradient A'w of ||.||_1 at x
end